unit MainForm;

interface

uses Windows, Messages, NForm, FormBg, Gauge, Element, Layout, LogDialog;

type
  TNkMonMainForm = class(TForm)
    MoveX, MoveY: Integer;
    Bg: TFormBg;
    Gauges: TGauges;
    Layouter: TLayouter;
    LogDlg: TLogDlg;
    Running: Boolean;
    constructor Create(const Caption: string; ALeft, ATop, AWidth,
      AHeight: Integer); override;
    destructor Destroy; override;
    procedure LoadConfig;
    procedure Log(Sender: TObject; const S: string);
    procedure Run;
    procedure Pause;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND;
    procedure WMContextMenu(var Message: TMessage); message WM_CONTEXTMENU;
    procedure WMLButtonDown(var Message: TWMLButtonDown);
      message WM_LBUTTONDOWN;
    procedure WMExitSizeMove(var Message: TMessage); message WM_EXITSIZEMOVE;
    procedure WMWinIniChange(var Message: TMessage); message WM_WININICHANGE;
    procedure WMMoving(var Message: TMessage); message WM_MOVING;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMTimer(var Message: TWMTimer); message WM_TIMER;
    procedure WMQueryEndSession(var Message: TWMQueryEndSession);
      message WM_QUERYENDSESSION;
    procedure WMClose(var Message: TWMClose); message WM_CLOSE;
  end;

var
  AppName, ProfDir: string;

implementation

uses NSys, NLib, NUtil, NConf;

{$INCLUDE version.inc}

const
  //IDM_EDIT = 101;
  IDM_RELOAD = 102;
  IDM_PAUSE = 103;
  IDM_LOG = 104;
  IDM_STAYONTOP = 105;
  IDM_ALLVALUE = 106;
  IDM_ABOUT = 108;
  IDM_EXIT = 109;

constructor TNkMonMainForm.Create(const Caption: string; ALeft, ATop, AWidth,
  AHeight: Integer);
begin
  inherited;
  LogDlg := TLogDlg.CreateByResource(Handle, 'LOGDIALOG');
  LogDlg.AppName := AppName;
  //SetBackground(GetStockObject(NULL_BRUSH));
  SetToolWindow(True);
  SetWindowLong(Handle, GWL_STYLE, Longint(WS_POPUP));
  //SetBorder(False);
  SetStayOnTop(True);
  LoadFormPos(ProfDir, Self);
  Gauges := TGauges.Create;
  Gauges.Log := Log;
  Layouter := TLayouter.Create;
  Bg := TFormBg.Create;
  Bg.Form := Self;
end;

destructor TNkMonMainForm.Destroy;
begin
  Pause;
  Layouter.Free;
  Gauges.Free;
  Bg.Free;
  LogDlg.Free;
  inherited;
end;

procedure TNkMonMainForm.Run;
begin
  SetTimer(Handle, 1, 1000, nil);
  Running := True;
  Log(Self, 'run');
end;

procedure TNkMonMainForm.Pause;
begin
  KillTimer(Handle, 1);
  Running := False;
  Log(Self, 'pause');
end;

procedure TNkMonMainForm.LoadConfig;
begin
  Gauges.Clear;
  try
    with TConf.Create(JoinPath(ProfDir, 'config')) do
      try
        while ReadLine do
          Gauges.Conf[GetArg(0)] := GetArg(1);
      finally
        Free;
      end;
  except
    on E: EIOError do ;
  end;
  Layouter.Load(JoinPath(ProfDir, 'layout'), Gauges);
  SetSize(Layouter.Width, Layouter.Height);
  Bg.R := Layouter.BgR;
  Bg.G := Layouter.BgG;
  Bg.B := Layouter.BgB;
  Bg.A := Layouter.BgA;
  Gauges.Update;
  Layouter.Update;
end;

procedure TNkMonMainForm.Log(Sender: TObject; const S: string);
var
  st: TSystemTime;
  from: string;
begin
  GetLocalTime(st);
  if Sender = Self then
    from := ''
  else
    from := Lower(Copy(Sender.ClassName, 2, Length(Sender.ClassName)-1))+': ';
  LogDlg.Write(Fmt('%02s-%02s %02s:%02s:%02s %s%s'+EOL,
    [st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, from, S]));
end;

procedure TNkMonMainForm.WMCommand(var Message: TWMCommand);
var
  Name: string;
  i: Integer;
begin
  case Message.ItemID of
    IDM_RELOAD:
      begin
        Pause;
        LoadConfig;
        Bg.Make;
        Run;
      end;
    IDM_PAUSE:
      if Running then
        Pause
      else
        Run;
    IDM_STAYONTOP: SetStayOnTop(not StayOnTop);
    IDM_ALLVALUE:
      with TStrList.Create do
        try
          while Gauges.EachSymbols(Name) do
            Add(Name);
          Sort(StrColl);
          for i := 0 to Count-1 do
            Log(Self, Strings[i]+'='+Gauges[Strings[i]]);
        finally
          Free;
        end;
    IDM_LOG:
      if not IsWindow(LogDlg.Handle) then
        LogDlg.ShowModeless;
    IDM_ABOUT:
      AboutBox(Handle, AppName,
        AppName+' '+Version+' ('+ReleaseDate+')'+EOL+EOL+
        Copyright+EOL+EOL+
        'This software is provided under the zlib/libpng license.'+EOL+EOL+
        'Profile Folder: '+ProfDir);
    IDM_EXIT: Close;
  end;
end;

procedure TNkMonMainForm.WMContextMenu(var Message: TMessage);
var
  x, y: Integer;
begin
  with TPopupMenu.Create do
    try
      AppendItem('&Pause', IDM_PAUSE);
      AppendItem('&Log Window', IDM_LOG);
      AppendItem('Show All &Values', IDM_ALLVALUE);
      AppendItem('-', 0);
      AppendItem('Stay &On Top', IDM_STAYONTOP);
      AppendItem('-', 0);
      AppendItem('&Reload Config and Layout', IDM_RELOAD);
      //AppendItem('&Edit Config', IDM_EDIT);
      AppendItem('-', 0);
      AppendItem('&About', IDM_ABOUT);
      AppendItem('-', 0);
      AppendItem('E&xit', IDM_EXIT);
      CheckItem(IDM_PAUSE, not Running);
      CheckItem(IDM_STAYONTOP, StayOnTop);
      if Message.LParam = -1 then
      begin
        ClientToScreen(0, 0, x, y);
        Popup(Self.Handle, x, y)
      end else
        Popup(Self.Handle, Message.LParamLo, Message.LParamHi);
    finally
      Free;
    end;
end;

procedure TNkMonMainForm.WMTimer(var Message: TWMTimer);
begin
  Gauges.Update;
  Layouter.Update;
  InvalidateRect(Handle, nil, False);
end;

procedure TNkMonMainForm.WMExitSizeMove(var Message: TMessage);
begin
  Bg.Make;
  Invalidate;
end;

procedure TNkMonMainForm.WMWinIniChange(var Message: TMessage);
begin
  Bg.Make;
  Invalidate;
end;

// XXX
procedure TNkMonMainForm.WMMoving(var Message: TMessage);
  procedure OffsetRect(rc: PRect; x, y: Integer);
  begin
    Inc(rc.Left, x);
    Inc(rc.Top, y);
    Inc(rc.Right, x);
    Inc(rc.Bottom, y);
  end;
const
  snap = 10;
var
  rc: PRect;
  drc: TRect;
  x, y, w, h: Integer;
begin
  GetBounds(x, y, w, h);
  rc := PRect(Message.LParam);
  Message.Result := LRESULT(True);
  GetWindowRect(GetDesktopWindow, drc);
  if Abs(drc.Left-rc.Left) < snap then
  begin
    Inc(MoveX, rc.Left-drc.Left);
    if Abs(MoveX) > snap then
    begin
      OffsetRect(rc, MoveX, 0);
      MoveX := 0;
    end else
      OffsetRect(rc, drc.Left-rc.Left, 0);
  end;
  if Abs(drc.Top-rc.Top) < snap then
  begin
    Inc(MoveY, rc.Top-drc.Top);
    if Abs(MoveY) > snap then
    begin
      OffsetRect(rc, 0, MoveY);
      MoveY := 0;
    end else
      OffsetRect(rc, 0, drc.Top-rc.Top);
  end;
  if Abs(drc.Right-rc.Right) < snap then
  begin
    Inc(MoveX, rc.Right-drc.Right);
    if Abs(MoveX) > snap then
    begin
      OffsetRect(rc, MoveX, 0);
      MoveX := 0;
    end else
      OffsetRect(rc, drc.Right-rc.Right, 0);
  end;
  if Abs(drc.Bottom-rc.Bottom) < snap then
  begin
    Inc(MoveY, rc.Bottom-drc.Bottom);
    if Abs(MoveY) > snap then
    begin
      OffsetRect(rc, 0, MoveY);
      MoveY := 0;
    end else
      OffsetRect(rc, 0, drc.Bottom-rc.Bottom);
  end;
end;

procedure TNkMonMainForm.WMPaint(var Message: TWMPaint);
var
  dc, mdc: HDC;
  bmp, oldbmp: HBITMAP;
  ps: TPaintStruct;
  w, h: Integer;
begin
  dc := BeginPaint(Handle, ps);
  mdc := CreateCompatibleDC(dc);
  GetClientSize(w, h);
  bmp := CreateCompatibleBitmap(dc, w, h);
  oldbmp := SelectObject(mdc, bmp);
  Bg.Paint(mdc);
  Layouter.Draw(mdc);
  BitBlt(DC, 0, 0, w, h, mdc, 0, 0, SRCCOPY);
  SelectObject(mdc, oldbmp);
  DeleteObject(bmp);
  DeleteDC(mdc);
  EndPaint(Handle, ps);
end;

procedure TNkMonMainForm.WMLButtonDown(var Message: TWMLButtonDown);
begin
  SendMsg(WM_NCLBUTTONDOWN, HTCAPTION, 0);
end;

procedure TNkMonMainForm.WMClose(var Message: TWMClose);
begin
  SaveFormPos(ProfDir, Self);
  inherited;
end;

procedure TNkMonMainForm.WMQueryEndSession(var Message: TWMQueryEndSession);
begin
  SaveFormPos(ProfDir, Self);
  Message.Result := Integer(True);
end;

end.
